%% Performing signal intensity normalization and signal drift correction

% Fuyixue Wang <fuyixue@mit.edu> or <fuyixue2016@gmail.com>, Aug/2020

clear;
close all;
%% data directory and filenames
dir_reconData = '../reconDWI_';
dir_tmp = '../tmp/';
fname = ["session1_subset01","session1_subset02","session2_subset03","session2_subset04","session3_subset05","session3_subset06",...
         "session4_subset07","session4_subset08","session5_subset09","session5_subset10","session6_subset11",'session6_subset12',...
         "session7_subset13","session7_subset14","session8_subset15","session8_subset16","session9_subset17",'session9_subset18'];
%%
N_data = length(fname);             % number of data subsets
N_b0 = 8;                           % number of b=0 in each acquisition block (AP or PA)
N_dwi = 10;                         % number of DWI between two b=0 s/mm2 images
Norder_interp = 2;                  % order of polynomial to fit signal drift
N_interp = (N_b0-1)*N_dwi + N_b0;   % number of volume in each acquisition bloc (AP or PA)

t_AP = zeros(N_b0,N_data);
t_PA = zeros(N_b0,N_data);
t_AP_interp = zeros(N_interp,N_data);
t_PA_interp = zeros(N_interp,N_data);

for n=1:length(fname)
    disp(['--------------',fname{n},'----------------']);
    fpBval = fullfile([dir_reconData,fname{n}],[fname{n},'.bval']);
    fpDiff = fullfile([dir_reconData,fname{n}],[fname{n},'.nii.gz']);
    
    dir_sigNorm = fullfile(dir_tmp,'sigNorm',fname{n});
    fpB0_AP = fullfile(dir_sigNorm, [fname{n},'_B0_AP']);
    fpB0_PA = fullfile(dir_sigNorm, [fname{n},'_B0_PA']);
    fpB0_AP_Tmean = fullfile(dir_sigNorm, [fname{n},'_B0_AP_Tmean']);
    fpB0_PA_Tmean = fullfile(dir_sigNorm, [fname{n},'_B0_PA_Tmean']);
    fpB0_AP_Tmean_masked = fullfile(dir_sigNorm, [fname{n},'_B0_AP_Tmean_bet']);
    fpB0_PA_Tmean_masked = fullfile(dir_sigNorm, [fname{n},'_B0_PA_Tmean_bet']);
    fpB0_AP_Tmean_mask = fullfile(dir_sigNorm, [fname{n},'_B0_AP_Tmean_bet_mask']);
    fpB0_PA_Tmean_mask = fullfile(dir_sigNorm, [fname{n},'_B0_PA_Tmean_bet_mask']);

    % extract b=0 images and generate brain masks
    mkdir(dir_sigNorm);
    mask_b0 = dlmread(fpBval) == 0;
    mask_b0_AP = cat(2,mask_b0(:,1:length(mask_b0)/2),zeros(1,length(mask_b0)/2));
    mask_b0_PA = cat(2,zeros(1,length(mask_b0)/2),mask_b0(:,length(mask_b0)/2+1:end));

    disp('Extracting AP b=0 images');
    fpList_AP = fullfile(dir_sigNorm, 'vol_AP.list');
    dlmwrite(fpList_AP, find(mask_b0_AP) - 1, 'delimiter', ' ', 'precision', 6);
    cmd = ['fslselectvols -i ' fpDiff ' -o ' fpB0_AP ' --vols=' fpList_AP];
    [status, result] = system(cmd, '-echo');

    disp('Extracting PA b=0 images');
    fpList_PA = fullfile(dir_sigNorm, 'vol_PA.list');
    dlmwrite(fpList_PA, find(mask_b0_PA) - 1, 'delimiter', ' ', 'precision', 6);
    cmd = ['fslselectvols -i ' fpDiff ' -o ' fpB0_PA ' --vols=' fpList_PA];
    [status, result] = system(cmd, '-echo');

    disp('generating mask ...');
    cmd = ['fslmaths ',fpB0_AP,' -Tmean ',fpB0_AP_Tmean];
    [status, result] = system(cmd, '-echo');

    cmd = ['bet2 ',fpB0_AP_Tmean,' ',fpB0_AP_Tmean_masked,' -f 0.1 -m'];
    [status,result] = system(cmd, '-echo');  

    cmd = ['fslmaths ',fpB0_PA,' -Tmean ',fpB0_PA_Tmean];
    [status, result] = system(cmd, '-echo');

    cmd = ['bet2 ',fpB0_PA_Tmean,' ',fpB0_PA_Tmean_masked,' -f 0.1 -m'];
    [status,result] = system(cmd, '-echo'); 

    % calculating signal intensity across time
    disp('calculating signal intensity ...');
    data_AP = load_nifti([fpB0_AP,'.nii.gz']);
    data_AP_mask = load_nifti([fpB0_AP_Tmean_mask,'.nii.gz']);

    data_PA = load_nifti([fpB0_PA,'.nii.gz']);
    data_PA_mask = load_nifti([fpB0_PA_Tmean_mask,'.nii.gz']);

    for dif_t = 1:N_b0
        temp_AP = data_AP.vol(:,:,:,dif_t);
        t_AP(dif_t,n) = mean(abs(temp_AP(logical(data_AP_mask.vol))));
        temp_PA = data_PA.vol(:,:,:,dif_t);
        t_PA(dif_t,n) = mean(abs(temp_PA(logical(data_PA_mask.vol))));
    end
    % fitting signal drift within each acquisition block
    t_AP_interp(:,n) = polyval( polyfit( 1:N_b0,t_AP(:,n)', Norder_interp), linspace(1,N_b0,N_interp) );
    t_PA_interp(:,n) = polyval( polyfit( 1:N_b0,t_PA(:,n)', Norder_interp), linspace(1,N_b0,N_interp) );
end

% signal normalization across sessions
t_AP_interp_norm = t_AP_interp./mean(t_AP_interp(:,2),1);
t_PA_interp_norm = t_PA_interp./mean(t_PA_interp(:,2),1);
t_APPA = cat(1,t_AP_interp_norm,t_PA_interp_norm);

% data signal intensity correction
for n=1:length(fname)
    fpDiff = fullfile([dir_reconData,fname{n}],[fname{n},'.nii.gz']);
    fpDiff_norm = fullfile(dir_tmp,'sigNorm',[fname{n},'_sigNorm.nii']);
    data = load_nifti(fpDiff);
    data.vol = data.vol./permute(repmat(t_APPA(:,n),[1,size(data.vol,1),size(data.vol,2),size(data.vol,3)]),[2,3,4,1]);
    save_nifti(data,fpDiff_norm);
end