%% Performing susceptibility and eddy-current induced distortion

% Fuyixue Wang <fuyixue@mit.edu> or <fuyixue2016@gmail.com>, Aug/2020

clear
close all
%% data directory and filenames
dir_reconData = '../reconDWI_';
dir_tmp = '../tmp/';
fname = ["session1_subset01","session1_subset02","session2_subset03","session2_subset04","session3_subset05","session3_subset06",...
         "session4_subset07","session4_subset08","session5_subset09","session5_subset10","session6_subset11",'session6_subset12',...
         "session7_subset13","session7_subset14","session8_subset15","session8_subset16","session9_subset17",'session9_subset18'];
N_b0_topup = 4;                   % number of B0 images for susceptibility field estimation
%%
for n = 1:length(fname)
    
    disp(['------------',fname{n},'--------------']);
    fpdata = fullfile(dir_tmp,'sigNorm',[fname{n},'_sigNorm.nii']);
    fpBvec =  fullfile([dir_reconData,fname{n}],[fname{n},'.bvec']);
    fpBval =  fullfile([dir_reconData,fname{n}],[fname{n},'.bval']);
    
    %%% topup settings
    dir_topup = fullfile(dir_tmp,'topup',fname{n});                  
    fpconfig_topup = fullfile(dir_tmp,'topup','b02b0_topup.cnf');   
    % topup input
    acqp_base0 = [0 1 0 0.1];
    acqp_base1 = [0 -1 0 0.1];
    fpB0_topup = fullfile(dir_topup, [fname{n},'_b0']);
    fpAcqp_topup = fullfile(dir_topup, [fname{n},'_acqp']);
    % topup output
    fpOut_topup = fullfile(dir_topup, [fname{n},'_topup_out']);
    fpIout_topup = fullfile(dir_topup, [fname{n},'_topup_b0_unwarp']);
    fpField_topup = fullfile(dir_topup, [fname{n},'_topup_field']);
    fpWarp_topup = fullfile(dir_topup, [fname{n},'_topup_warpfield']);

    fpIout_topup_mean = fullfile(dir_topup,[fname{n},'_topup_b0_unwarp_TmeanImg']);
    fpIout_topup_masked = fullfile(dir_topup,[fname{n},'_topup_b0_unwarp_TmeanImg_bet']);
    fpIout_topup_mask = fullfile(dir_topup,[fname{n},'_topup_b0_unwarp_TmeanImg_bet_mask']);

    %%% Eddy settings
    dir_eddy = fullfile(dir_tmp,'eddy',fname{n});
    % eddy input
    fpAcqp_eddy = fullfile(dir_eddy, [fname{n},'_acqp']);
    fpIndex_eddy = fullfile(dir_eddy, [fname{n},'_indx']);
    % eddy output
    fpOut_eddy =  fullfile(dir_eddy,[fname{n},'_eddy']);
    
    %% performing topup
    mkdir(dir_topup);
    % generating b0 images for topup
    disp(['Taking out the first ',num2str(N_b0_topup),' pairs of B0 AP-PA images...']);
    mask_b0 = dlmread(fpBval) == 0;
    indx = find(mask_b0==1,N_b0_topup+1);
    mask_b0(indx(end):end) = 0;
    mask_b0 = cat(2,mask_b0(:,1:length(mask_b0)/2),mask_b0(:,1:length(mask_b0)/2)); % for AP-PA

    fpList = fullfile(dir_topup, 'vol.list');
    dlmwrite(fpList, find(mask_b0) - 1, 'delimiter', ' ', 'precision', 6);
    cmd = ['fslselectvols -i ' fpdata ' -o ' fpB0_topup ' --vols=' fpList];
    [status, result] = system(cmd, '-echo');

    % writing acqp files  
    disp('Writing acqp file...');
    acqp0_topup = repmat(acqp_base0, [N_b0_topup, 1]);
    acqp1_topup = repmat(acqp_base1, [N_b0_topup, 1]);
    acqp_topup = [acqp0_topup; acqp1_topup];
    dlmwrite(fpAcqp_topup, acqp_topup, 'delimiter', ' ', 'precision', 6);

    % running topup
    disp('running topup');
    tic
    cmd = ['topup',...
               ' --imain=',fpB0_topup,...
               ' --config=',fpconfig_topup,...
               ' --datain=',fpAcqp_topup,...
               ' --out=',fpOut_topup,...
               ' --iout=',fpIout_topup,...
               ' --fout=',fpField_topup,...
               ' --dfout=',fpWarp_topup];
    [status, result] = system(cmd, '-echo');
    toc
    disp('Done topup!');

    % mask generating
    disp('generating mask ...');
    cmd = ['fslmaths ',fpIout_topup,' -Tmean ',fpIout_topup_mean];
    [status, result] = system(cmd, '-echo');

    cmd = ['bet2 ',fpIout_topup_mean,' ',fpIout_topup_masked,' -f 0.1 -m'];
    [status,result] = system(cmd, '-echo');  
    
    %% performing eddy
    mkdir(dir_eddy);
    % writing diffusion/indexing files
    disp('writing diffusion and indexing files ...');
    acqp_eddy = [acqp_base0; acqp_base1];
    dlmwrite(fpAcqp_eddy, acqp_eddy, 'delimiter', ' ', 'precision', 6);

    index_eddy = ones(size(dlmread(fpBval)));
    index_eddy (:,length(index_eddy)/2+1:end) = 2;
    dlmwrite(fpIndex_eddy, index_eddy, 'delimiter', ' ', 'precision', 6);

    % running eddy in eddy_cuda
    cmd = ['eddy_cuda9.1',...
        ' --imain=',fpdata,...
        ' --mask=',fpIout_topup_mask,...
        ' --acqp=',fpAcqp_eddy,...
        ' --index=',fpIndex_eddy,...
        ' --bvecs=' fpBvec,...
        ' --bvals=' fpBval,...
        ' --topup=',fpOut_topup,...
        ' --out=' fpOut_eddy,...
        ' --dfields',...
        ' --flm=cubic --interp=spline --niter=8',...
        ' --repol --estimate_move_by_susceptibility --dont_peas -v'];
    [status,result] = system(cmd, '-echo');  
end