%% Post eddy motion estimation using b=0 s/mm2 images

% Fuyixue Wang <fuyixue@mit.edu> or <fuyixue2016@gmail.com>, Aug/2020

clear
close all
%% data directory and filenames
dir_tmp = '../tmp/';
dir_eddy = fullfile(dir_tmp,'eddy');

dir_b0 = fullfile(dir_tmp,'postEddy','b0');
dir_gnc = fullfile(dir_tmp,'postEddy','gnc');
dir_bet = fullfile(dir_tmp,'postEddy','bet');
dir_mov = fullfile(dir_tmp,'postEddy','mov');

fpGncfield = fullfile(dir_tmp,'private','deform_grad_rel');  % not provided, proprietary information by Siemens
fpFlirtRef = fullfile(dir_bet,'session1_subset01_gnc_bet');

fname = ["session1_subset01","session1_subset02","session2_subset03","session2_subset04","session3_subset05","session3_subset06",...
         "session4_subset07","session4_subset08","session5_subset09","session5_subset10","session6_subset11",'session6_subset12',...
         "session7_subset13","session7_subset14","session8_subset15","session8_subset16","session9_subset17",'session9_subset18'];
%%
mkdir(dir_b0);
mkdir(dir_gnc);
mkdir(dir_bet);
mkdir(dir_mov);

for n = 1:length(fname)
    disp(['------------',fname{n},'--------------']);
    fpOut_eddy =  fullfile(dir_eddy,fname{n},[fname{n},'_eddy']);
    fpB0 = fullfile(dir_b0,fname{n});
    fpB0_gnc = fullfile(dir_gnc,[fname{n},'_gnc']);
    fpB0_gnc_bet = fullfile(dir_bet,[fname{n},'_gnc_bet']);
    fpB0_gnc_bet_mov = fullfile(dir_mov,[fname{n},'_gnc_bet_mov']);
    fpAffineMat = fullfile(dir_mov,['AffMat_',fname{n},'.mat']);

    disp('taking out b0 roi ...');    
    cmd = ['fslroi ',fpOut_eddy,' ',fpB0,' 0 1'];
    [status, result] = system(cmd, '-echo');

    disp('gradient nonlinearity correction ...');
    cmd = ['applywarp -i ',fpB0,' -r ',fpB0,' -o ',fpB0_gnc,' -w ',fpGncfield,' --interp=spline --datatype=float --rel'];
    [status, result] = system(cmd, '-echo');

    disp('bet brain masking ...');
    cmd = ['bet2 ',fpB0_gnc,' ',fpB0_gnc_bet,' -f 0.008'];
    [status,result] = system(cmd, '-echo'); 

    disp('motion correction ...');
    cmd = ['flirt -in ',fpB0_gnc_bet,' -ref ',fpFlirtRef,' -out ',fpB0_gnc_bet_mov,' -omat ',fpAffineMat,' -dof 6 -interp spline -cost normcorr'];
    [status,result] = system(cmd, '-echo');  
end