%% Performing one-step resampling to avoid potential blurring due to multiple interpolation

% Fuyixue Wang <fuyixue@mit.edu> or <fuyixue2016@gmail.com>, Aug/2020

clear
close all
%% data directory and filenames
dir_tmp = '../tmp/';
fname = ["session1_subset01","session1_subset02","session2_subset03","session2_subset04","session3_subset05","session3_subset06",...
         "session4_subset07","session4_subset08","session5_subset09","session5_subset10","session6_subset11",'session6_subset12',...
         "session7_subset13","session7_subset14","session8_subset15","session8_subset16","session9_subset17",'session9_subset18'];
N_volume = 156;                                                     % total number of volumes for each data subet
fpGncfield = fullfile(dir_tmp,'private','deform_grad_rel');         % not provided, proprietary information by Siemens

for n = 1:length(fname)
    dir_eddy = fullfile(dir_tmp,'eddy',fname{n});
    dir_splitData = fullfile(dir_tmp,'splitdata',fname{n});
    dir_resampData = fullfile(dir_tmp,'resampdata',fname{n});

    fpData = fullfile(dir_eddy,[fname{n},'_eddy.eddy_outlier_free_data']);
    fpEddyField = fullfile(dir_eddy,[fname{n},'_eddy.eddy_displacement_fields.']);
    fpAffMat = fullfile(dir_tmp,'postEddy','mov',['AffMat_',fname{n},'.mat']);
    fpSplitData = fullfile(dir_splitData,[fname{n},'_']);
    fpCombField = fullfile(dir_splitData,['combField_',fname{n},'_']);
    fpJacobian = fullfile(dir_splitData,['jacobian_',fname{n},'_']);
    fpResampledData = fullfile(dir_resampData,[fname{n},'_']); 
    %%        
    mkdir(dir_splitData);
    mkdir(dir_resampData);

    disp(['spliting data ',fname{n}]);
    cmd = ['fslsplit ',fpData,' ',fpSplitData,' -t'];
    [status, result] = system(cmd, '-echo');
    
    for dif_t = 1:N_volume  
        disp(['convertwarp and applywarp of ',fname{n},': t',num2str(dif_t)])
        cmd = ['convertwarp -o ',[fpCombField,sprintf('%03d',dif_t)],... 
                          ' -r ',fpGncfield,...
                          ' --postmat=',fpAffMat,...
                          ' --warp1=',[fpEddyField,sprintf('%03d',dif_t)],...
                          ' --warp2=',fpGncfield,...
                          ' --rel --absout -v']; % relative
        [status, result] = system(cmd, '-echo');

        cmd = ['applywarp -i ',[fpSplitData,sprintf('%04d',dif_t-1)],...
                        ' -r ',[fpSplitData,sprintf('%04d',0)],...
                        ' -o ',[fpResampledData,sprintf('%03d',dif_t)],...
                        ' -w ',[fpCombField,sprintf('%03d',dif_t)],...
                        ' --interp=spline --datatype=float --abs'];
        [status, result] = system(cmd, '-echo');

        data = load_nifti([fpCombField,sprintf('%03d',dif_t),'.nii.gz']);
        jacdet = evaluate_jacobian(data.vol,data.pixdim(2:4));
        data.vol = jacdet;
        data.dim(1) = 3;
        data.dim(5) = 1;
        save_nifti(data,[fpJacobian,sprintf('%03d',dif_t),'.nii']);

        cmd = ['fslmaths ',fpResampledData,sprintf('%03d',dif_t),...
                 ' -mul ',fpJacobian,sprintf('%03d',dif_t),...
                 ' ',fpResampledData,sprintf('%03d',dif_t)];
        [status, result] = system(cmd, '-echo');
    end
    
    disp(['fslmerge ',fname{n},'...'])
    cmd = ['fslmerge -t ',dir_resampData,' ',dir_resampData,'/*']; % this is the final preprocessed data saved in processedDWI_sessionN_subsetM/sessionN_subsetM.nii.gz
    [status, result] = system(cmd, '-echo');

    cmd = ['rm -r ',dir_resampData];
    [status, result] = system(cmd, '-echo');
end

%% functions
function jacdet = evaluate_jacobian(field_data,pixdim)
    siemens_max_det = 10;
    [dFxdy, dFxdx, dFxdz] = gradient(field_data(:,:,:,1), pixdim(1), pixdim(2), pixdim(3));
    [dFydy, dFydx, dFydz] = gradient(field_data(:,:,:,2), pixdim(1), pixdim(2), pixdim(3));
    [dFzdy, dFzdx, dFzdz] = gradient(field_data(:,:,:,3), pixdim(1), pixdim(2), pixdim(3));

    jacdet = abs(dFxdx.*dFydy.*dFzdz-dFxdx.*dFydz.*dFzdy-dFxdy.*dFydx.*dFzdz+dFxdy.*dFydz.*dFzdx+dFxdz.*dFydx.*dFzdy-dFxdz.*dFydy.*dFzdx);
    jacdet(jacdet > siemens_max_det) = siemens_max_det;    
end